@@ -38,7 +38,7 @@ subroutine max_val_1d(var, mamn, mamv, undef)  ! 1 Ѻ, ֹ롼
      end do
 
      if(undeflag.eqv..true.)then
-        mamn=int(undef)
+        mamn=nx+1
         mamv=undef
      end if
 
@@ -81,14 +81,23 @@ subroutine max_val_2d(var, mamnx, mamny, mamv, undef)  ! 2 Ѻ, ֹ롼
         call max_val_1d(var(:,j),tmp_num(j),tmp(j),undef)
      end do
      call max_val_1d(tmp,mamny,mamv,undef)
+
+     if(mamny==ny+1)then
+        mamnx=nx+1
+     else
+        mamnx=tmp_num(mamny)
+     end if
+
   else
+
      do j=1,ny
         call max_val_1d(var(:,j),tmp_num(j),tmp(j))
      end do
      call max_val_1d(tmp,mamny,mamv)
-  end if
 
-  mamnx=tmp_num(mamny)
+     mamnx=tmp_num(mamny)
+
+  end if
 
 end subroutine max_val_2d
 
@@ -119,15 +128,26 @@ subroutine max_val_3d(var, mamnx, mamny, mamnz, mamv, undef)  ! 3 Ѻ, ֹ롼
         call max_val_2d(var(:,:,j),tmp_num1(j),tmp_num2(j),tmp(j),undef)
      end do
      call max_val_1d(tmp,mamnz,mamv,undef)
+
+     if(mamnz==nz+1)then
+        mamnx=nx+1
+        mamny=ny+1
+     else
+        mamnx=tmp_num1(mamnz)
+        mamny=tmp_num2(mamnz)
+     end if
+
   else
+
      do j=1,nz
         call max_val_2d(var(:,:,j),tmp_num1(j),tmp_num2(j),tmp(j))
      end do
      call max_val_1d(tmp,mamnz,mamv)
-  end if
 
-  mamnx=tmp_num1(mamnz)
-  mamny=tmp_num2(mamnz)
+     mamnx=tmp_num1(mamnz)
+     mamny=tmp_num2(mamnz)
+
+  end if
 
 end subroutine max_val_3d
 
@@ -165,7 +185,7 @@ subroutine min_val_1d(var, mimn, mimv, undef)
      end do
 
      if(undeflag.eqv..true.)then
-        mimn=int(undef)
+        mimn=nx+1
         mimv=undef
      end if
 
@@ -183,11 +203,9 @@ subroutine min_val_1d(var, mimn, mimv, undef)
 
   end if
 
-
 end subroutine min_val_1d
 
 
-
 !-- 2 Ǥ, x  y  1 롼Ƥ, Ǥκͤ1
 !-- Ǽ, Ф1롼Ѥ뤳Ȥ, ΤκͤФ
 subroutine min_val_2d(var, mimnx, mimny, mimv, undef)  ! 2 ѺǾ, ֹ롼
@@ -211,14 +229,22 @@ subroutine min_val_2d(var, mimnx, mimny, mimv, undef)  ! 2 ѺǾ, ֹ롼
         call min_val_1d(var(:,j),tmp_num(j),tmp(j),undef)
      end do
      call min_val_1d(tmp,mimny,mimv,undef)
+
+     if(mimny==ny+1)then
+        mimnx=nx+1
+     else
+        mimnx=tmp_num(mimny)
+     end if
+
   else
+
      do j=1,ny
         call min_val_1d(var(:,j),tmp_num(j),tmp(j))
      end do
      call min_val_1d(tmp,mimny,mimv)
-  end if
 
-  mimnx=tmp_num(mimny)
+     mimnx=tmp_num(mimny)
+  end if
 
 end subroutine min_val_2d
 
@@ -250,15 +276,25 @@ subroutine min_val_3d(var, mimnx, mimny, mimnz, mimv, undef)  ! 3 ѺǾ, ֹ롼
         call min_val_2d(var(:,:,j),tmp_num1(j),tmp_num2(j),tmp(j),undef)
      end do
      call min_val_1d(tmp,mimnz,mimv,undef)
+
+     if(mimnz==nz+1)then
+        mimnx=nx+1
+        mimny=ny+1
+     else
+        mimnx=tmp_num1(mimnz)
+        mimny=tmp_num2(mimnz)
+     end if
+
   else
+
      do j=1,nz
         call min_val_2d(var(:,:,j),tmp_num1(j),tmp_num2(j),tmp(j))
      end do
      call min_val_1d(tmp,mimnz,mimv)
-  end if
 
-  mimnx=tmp_num1(mimnz)
-  mimny=tmp_num2(mimnz)
+     mimnx=tmp_num1(mimnz)
+     mimny=tmp_num2(mimnz)
+  end if
 
 end subroutine min_val_3d
 
