@@ -993,14 +993,12 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
                     v(i,j)=rdefun
                  end if
               else
-write(*,*) "kokonihakonaihazu1", x(nx), r(i), y(ny), q(j), ir, nx, iq, ny
                  v(i,j)=real(defun)
               end if
            else
               v(i,j)=rdefun
            end if
         else
-write(*,*) "kokonihakonaihazu2", x(nx), r(i), y(ny), q(j), ir, nx, iq, ny
            v(i,j)=real(defun)
         end if
      end do
@@ -1177,7 +1175,7 @@ subroutine LSM_1d( x, y, slope, intercept, undef )  ! Ǿˡˤ뷹ҷ׻ (1 ǡ)
   d=0.0
 
 !$omp parallel default(shared)
-!$omp do private(i)
+!$omp do schedule(runtime) private(i)
   do i=1,nx
      u(i)=x(i)*x(i)
      v(i)=x(i)*y(i)
